#include "huc.h"

#define FILE_PER_PAGE 10
#define MENU_Y_OFFSET 8

/*
const char bram_data[] = {
 0X0A, 0X00, 0X55, 0X55, 0XFF, 0XFF, 0X41, 0X55,
0X54, 0X54, 0X45, 0X20, 0X54, 0X49, 0X54, 0X52,
0X30, 0X31, 0X32, 0X33, 0X34, 0X35, 0X36, 0X37,
0X38, 0X39, 0XFF,
 0X00, 0X00
 };
*/

#include "filedata.dat"

#include "bramutil.c"

strncmp(lhs,rhs,size)
 char* lhs;
 char* rhs;
 int size;
{
	int index;
	index = 0;
	
	if (size-- == 0)
		return 0;
	
	while (size--)
	{
		if (lhs[index]!=rhs[index])
			return lhs[index] - rhs[index];
		index++;
	}

	return lhs[index] - rhs[index];

}

var_rawread_16(offset)
     int offset;
{
  return bram_data[offset] + (bram_data[offset + 1] << 8);
}

char var_file_name[11];
int var_file_size;
int var_file_crc;
int var_file_xx;
int var_file_data;
int var_file_pointer;

var_init_file_list()
{
  char i;
  for (i = 0; i < 11; i++)
    var_file_name[i] = 0;
  var_file_pointer = 0;
}

var_next_file()
{
  char i;

  var_file_size = var_rawread_16(var_file_pointer);

  if (var_file_size == 0)
    return BM_END_CHUNK_SIZE; /* abnormal end of bram, only the chunk size
				 reports so (we can't continue to parse)
			      */
  var_file_crc = var_rawread_16(var_file_pointer + 2);
  var_file_xx = var_rawread_16(var_file_pointer + 4);
  for (i = 0; i < 10; i++)
    var_file_name[i] = bram_data[i + var_file_pointer + 6];
  var_file_data = var_file_pointer + 0x10;
  var_file_pointer += var_file_size;
  return BM_CONTINUE;
}

var_file_nb(nb)
     int nb;
{
  char error;
  var_init_file_list();
  error = var_next_file();
  while ((nb--) && (error == BM_CONTINUE))
    error = var_next_file();
  return error;
}

var_total_file()
{
  int size;
  var_init_file_list();
  size = 0;
  while (var_next_file() == BM_CONTINUE)
    size++;
  return size;
}

menu_disp(choice)
     int choice;
{
  int i,j,page;
  char error;

  page = choice / FILE_PER_PAGE;

  error = var_file_nb(i = page * FILE_PER_PAGE);

  for (j=0; j <FILE_PER_PAGE; i++, j++)
    {

      if (error != BM_CONTINUE)
      {
	put_string("  --ooOOoo--            ", 0, j + MENU_Y_OFFSET);
	continue;
      }
      else
      {
	put_string(var_file_name, 2, j + MENU_Y_OFFSET);
	put_char('(', 13, j + MENU_Y_OFFSET);
	put_char(')', 18, j + MENU_Y_OFFSET);
	put_hex(var_file_xx, 4, 14, j + MENU_Y_OFFSET);
	put_hex(var_file_size, 4, 20, j + MENU_Y_OFFSET);
      }

      if (i == choice)
	put_string(">", 0, j + MENU_Y_OFFSET);
      else
	put_string(" ", 0, j + MENU_Y_OFFSET);

      error = var_next_file();
    }
}

show_header()
{
	put_string("File restoration", 10, 1);
	put_number(var_total_file(), 2, 3, 3);
	put_string("files(s) on CD", 6, 3);
	put_number(total_file(), 2, 3, 4);
	put_string("files(s) on BRAM", 6, 4);
	put_string("Name    ( ID ) Size", 5, 6);
}

restore_file()
{
	int index, size_bram, dest;
	char same_file, error;

	
	same_file = 0;
	size_bram = total_file();

	cls();
	init_file_list();

	for (index = 0; index < size_bram; index++)
	{
		/* char x,y,z,a,b,c; */ /* <= uncommenting this produces side effects */
		
		error = next_file();
		
		if (error != BM_CONTINUE)
		{
			put_string("* Error while parsing the bram *", 1, 2);
			while (!joytrg(0)) vsync(1);
			return;
		 }
				 
		 if (strncmp(var_file_name, file_name, 10) != 0)
			 continue;
		 
		if (var_file_xx == file_xx)
		{
			same_file = 1;
			break;
		}			
		
	}

	if (same_file != 0)
	{
		put_string("** Existing file of this name **", 0, 2);
		put_string("** already in your bram **", 3, 3);
		vsync(5);
		while (!joytrg(0)) vsync(1);
		return;
	}	
	
	/* ok, here, we're sure there's no file of the same name in the bram */
	
	put_string("- File not yet in bram", 2, 10);
	
	if (free_space() == 0xFFFF)
	{
		put_string("**        Corrupted bram      **", 0, 2);
		put_string("**  Fix or reformat it **", 3, 3);
		vsync(5);
		while (!joytrg(0)) vsync(1);
		return;		
	}
	
	put_string("- Bram sane", 2, 11);
	
	if (free_space() < 2 + var_file_size) /* finals 0 (on 16 bits) + raw data */
	{
		put_string("** Not enough free space **", 0, 2);
		put_number(2+16+var_file_size, 4, 5, 3);
		put_number("needed", 7, 3);
		put_number(free_space(), 4, 5, 4);
		put_number("available", 7, 4);
		vsync(5);
		while (!joytrg(0)) vsync(1);
		return;
	}

	put_string("- Free space sufficient", 2, 12);


	dest = bm_rawread_16(6);

	bm_rawwrite(dest++, var_file_size & 0xFF); /* writing file size */
	bm_rawwrite(dest++,var_file_size >> 8);

	bm_rawwrite(dest++, var_file_crc & 0xFF); /* writing file crc */
	bm_rawwrite(dest++, var_file_crc >> 8);

	bm_rawwrite(dest++, var_file_xx & 0xFF); /* writing file ID */
	bm_rawwrite(dest++, var_file_xx >> 8);

	for (index = 0; index < 10; index++)
		bm_rawwrite(dest++, var_file_name[index]); /* writing file name */

	for (index = 0; index < var_file_size - 0x10; index++) /* header is already written */
	{
		bm_rawwrite(dest++, bram_data[var_file_data + index]); /* writing raw data */
		put_hex(bram_data[var_file_data + index], 2, 3, index);
		put_hex(var_file_data + index , 4, 7, index);
	}

	/* warning : var_file_pointer is now messed up, var_init_file_list,e.g., is needed to reinit it */

	/* dest should now point to the next free byte */

	bm_rawwrite(6, dest & 0xFF); /* Updating the bram header */
	bm_rawwrite(7, dest >> 8);

	bm_rawwrite(dest++, 0); /* Set the "end of parsing" token */
	bm_rawwrite(dest, 0);

	put_string("- SUCCESS !!", 2, 13);
	vsync(5);
	while (!joytrg(0)) vsync(1);
	
}

#defpal(bw_col,0x000, 0x777);

main ()
{
	char joy_status;
	int menu_current_choice, menu_max_choice;

  set_screen_size(SCR_SIZE_32x32);
  set_font_pal(0);
  set_font_color(1,0);
  set_bgpal(0, bw_col);
  load_default_font();
	cls();

  bm_end = bm_rawread_16(4) - 0x8000;

	menu_max_choice = var_total_file();
	menu_current_choice = 0;

	show_header();

    for (;;)
    {
      menu_disp(menu_current_choice);

      do {
	vsync(1);
	joy_status = joytrg(0);
      } while (!joy_status);

      if (joy_status & JOY_DOWN)
	{
	if (menu_current_choice < menu_max_choice - 1)
	  menu_current_choice++;
	}
      else if (joy_status & JOY_UP)
	{
	  if (menu_current_choice != 0)
	    menu_current_choice--;
	}
      else if (joy_status & JOY_A)
	{
	  var_file_nb(menu_current_choice);
	  cls();
	  restore_file();
	  cls();
	  show_header();
	  var_init_file_list();
	}
      #ifdef CD_VERSION
      else if (joy_status & JOY_B)
	cd_execoverlay(OVL_MENU);
      #endif
    }

	
}